/**
 * Original From @hidoo/gulp-plugin-image-placeholder
 * forked and modifed by SimplyLouis
 *
 * Replace all image by default
 */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = _interopRequireDefault(require("path"));

var _through = _interopRequireDefault(require("through2"));

var _vinyl = _interopRequireDefault(require("vinyl"));

var _pluginError = _interopRequireDefault(require("plugin-error"));

var _getStream = _interopRequireDefault(require("get-stream"));

var _ndarray = _interopRequireDefault(require("ndarray"));

var _savePixels = _interopRequireDefault(require("save-pixels"));

var _imageSize = _interopRequireDefault(require("image-size"));

var _fancyLog = _interopRequireDefault(require("fancy-log"));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);
    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }
    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }
  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

/**
 * plugin name.
 * @type {String}
 */
const PLUGIN_NAME = 'gulp-plugin-image-placeholder';
/**
 * plugin default options.
 * @type {Object}
 */

const DEFAULT_OPTIONS = {
  append: false,
  suffix: null,
  verbose: true
};

/**
 * return placeholder image.
 * @param {Object} options option
 * @param {Boolean} [options.append=true] append placeholder or not
 * @param {String} [options.suffix='placeholder'] placeholder image suffix
 * @param {Boolean} [options.verbose=false] out log or not
 * @return {DestroyableTransform}
 *
 * @example
 * import {src, dest, task} from 'gulp';
 * import imagePlaceholder from '@hidoo/gulp-plugin-image-placeholder';
 *
 * task('placeholder', () => src('/path/to/src')
 *   .pipe(imagePlaceholder({
 *     append: false,
 *     suffix: 'placehold',
 *     verbose: true
 *   }))
 *   .pipe(dest('/path/to/dest'))):
 */

function imagePlaceholder(options = {}) {
  const opts = _objectSpread({}, DEFAULT_OPTIONS, options);

  return _through.default.obj(function (file, enc, done) {
    if (file.isStream()) {
      throw new _pluginError.default(PLUGIN_NAME, 'Stream is not support.');
    }

    if (file.isNull()) {
      return done(null, file);
    }

    if (!file.isBuffer()) {
      return done(null, file);
    }

    const fileExt = file.extname.replace('.', '');

    if (['jpg', 'jpeg', 'gif', 'svg', 'png'].indexOf(fileExt) == -1) return done(null, file);

    const stream = this,
      // eslint-disable-line no-invalid-this, consistent-this
      {
        dirname,
        contents
      } = file,
      basename = _path.default.basename(file.path, file.extname),
      {
        width,
        height
      } = (0, _imageSize.default)(contents),
      saveOptions = {},
      fillPixel = [0, 0, 0, 0],
      channels = 4; // set params (example when image size is 12x10)
    // + shape -> [12, 10, 4]
    // + stride -> [4, 48, 1]


    const sizeInRow = width * channels,
      shape = [width, height, channels],
      stride = [channels, sizeInRow, 1]; // transform array to ndarray

    const data = Array(width * height).fill(fillPixel),
      pixels = (0, _ndarray.default)(Uint8ClampedArray.from(data), shape, stride);
    return _getStream.default.buffer((0, _savePixels.default)(pixels, fileExt, saveOptions)).then(buffer => {
      const suffix = opts.suffix ? `.${opts.suffix}` : '',
        placeholder = new _vinyl.default({
          cwd: file.cwd,
          base: file.base,
          path: `${dirname}/${basename}${suffix}.${fileExt}`,
          contents: buffer
        }); // append placeholder image after original image.

      if (opts.append) {
        if (opts.verbose) {
          (0, _fancyLog.default)(`${PLUGIN_NAME}: append ${basename}${suffix}.${fileExt}.`);
        }

        stream.push(file);
        stream.push(placeholder);
      } // push placeholder image only.
      else {
        stream.push(placeholder);
      }

      return done();
    }).catch(error => done(new _pluginError.default(PLUGIN_NAME, error)));
  });
}

module.exports.default = imagePlaceholder;

